/*===========================================================================

FILE: TapiExample.c
===========================================================================*/


/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
/*
#include "AEEModGen.h"          // Module interface definitions
#include "AEEAppGen.h"          // Applet interface definitions
#include "AEEShell.h"           // Shell interface definitions
#include "AEETapi.h"			// TAPI Interface definitions

#include "AEEStdLib.h"  // AEE StdLib Services
#include "AEEMenu.h"    // AEE Menu Services
#include "AEEDisp.h"		// AEE Display Services
*/
#include "AEEAppGen.h"	    // AEEApplet declaration
#include "AEEUsageAppIDs.h" // Class IDs of usage apps (including this one)
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	    // AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services

// Services used by this app
#include "AEETapi.h"		//	AEETapi Services
#include "AEEMenu.h"		// Menu Services
#include "AEEStdLib.h"      // AEE StdLib services
#include "AEEFile.h"        // File Services

#include "tapiexample.bid"

typedef enum
{
	EXAMPLE_NORMAL,
	EXAMPLE_SMS
} ExampleStatus;

typedef struct _CTapiExample {
	AEEApplet	        a;
	IMenuCtl *          m_pIMenu;
   	AEEDeviceInfo		m_dInfo;
	int					m_nLineHeight;
	char                smsbuffer[100];
	/*   Ѵ. */
	ExampleStatus		status;
} CTapiExample;

#define EVT_CALL_ARRIVED        EVT_USER + 0x2

#define MENU_EVT_MAKEVOICECALL	401
#define SMS_RECEIVE				402

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static void DisplayMenu(CTapiExample *pMe);
static boolean TapiExample_InitAppData(IApplet* pi);
static void TapiExample_FreeAppData(IApplet* pi);
static void DisplayOutput(IApplet * pi, int nline, char *pszStr, RGBVAL clrColor);
static boolean TapiExample_HandleEvent(IApplet * pi, AEEEvent eCode, 
                                      uint16 wParam, uint32 dwParam);

/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDisplay_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDisplay_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   if(ClsId == AEECLSID_TAPIEXAMPLE){
      if(AEEApplet_New(sizeof(CTapiExample), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)TapiExample_HandleEvent,(PFNFREEAPPDATA)TapiExample_FreeAppData)
         == TRUE)
      {
		 // Add your code here .....
		 if (TapiExample_InitAppData((IApplet*)*ppObj) == TRUE)
				return (AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION TapiExample_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean TapiExample_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean TapiExample_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
	CTapiExample* pMe = (CTapiExample*)pi;
	switch (eCode) 
	{
		case EVT_APP_START:                        
			{	
				DisplayMenu(pMe);
				
				if(pMe->status == EXAMPLE_SMS)
				{
					IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
					IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
					IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
					DisplayOutput((IApplet *)pMe, 4, pMe->smsbuffer , MAKE_RGB(0xff,0, 0));
					pMe->status = EXAMPLE_NORMAL;
					return TRUE;
				}
				return TRUE;
			}
		case EVT_KEY:
			if (pMe->m_pIMenu)
			{
				if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) && (wParam == AVK_RIGHT))
				{
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw(pMe->m_pIMenu);
				}
			    return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
			}
			else
				return FALSE;
		case EVT_APP_STOP:
			{
				return TRUE;
			}
		/* SMS ޽   EVT_APP_MESSAGE ̺Ʈ ޴´ */
		case EVT_APP_MESSAGE:
			{
				AEECLSID clsid;
				pMe->status = EXAMPLE_SMS;
				STRCPY(pMe->smsbuffer, (char*)dwParam);
				//MEMCPY(pMe->smsbuffer,(char *)dwParam, STRLEN((char *)dwParam));
				clsid = ISHELL_ActiveApplet(pMe->a.m_pIShell);
				
				if(clsid == AEECLSID_TAPIEXAMPLE)
				{
					ISHELL_SendEvent (pMe->a.m_pIShell, AEECLSID_TAPIEXAMPLE, EVT_USER, SMS_RECEIVE, 0);
					pMe->status = EXAMPLE_NORMAL;
				}
				else
					ISHELL_StartApplet(pMe->a.m_pIShell, AEECLSID_TAPIEXAMPLE);
			}
		case EVT_APP_SUSPEND:
            return(TRUE);
		case EVT_APP_RESUME:
            if (pMe->m_pIMenu)
            {
				IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
				IMENUCTL_Redraw(pMe->m_pIMenu);
			}
            return TRUE;
		case EVT_COMMAND:
			{
				switch(wParam)
				{
				case MENU_EVT_MAKEVOICECALL:
					{
						int retValue = 0;
						ITAPI * pITapi;
						// Create an instance of the ITapi interface
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						DBGPRINTF("START");
						if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_TAPI,(void **)&pITapi) != SUCCESS)
							return TRUE;
						DBGPRINTF("START-1");
						retValue = ITAPI_MakeVoiceCall(pITapi, "114", AEECLSID_TAPIEXAMPLE);
						switch(retValue)
						{
						case SUCCESS:
							DBGPRINTF("SUCCESS");
							break;
						case EBADPARM:
							DBGPRINTF("EBADPARM");
							break;
						case EALREADY:
							DBGPRINTF("EALREADY");
							break;
						default:
							break;

						}
						// Free the instance of the ITapi interface
						ITAPI_Release(pITapi);
						return TRUE;
					}
				case SMS_RECEIVE:
					{
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						DisplayOutput((IApplet *)pMe, 4, pMe->smsbuffer , MAKE_RGB(0xff,0, 0));
						return TRUE;
					}
				default:
					return TRUE;
				}
			}
		default:
			break;
   }
   return FALSE;
}


static void DisplayMenu(CTapiExample *pMe)
{
	AEERect rc;
	AECHAR  szBuf[50];
	AEEMenuColors mclr;
	
	if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu) != SUCCESS)
		return;
	
	// Set display rectangle
	SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

	// õ ޴  Ѵ. 
	mclr.wMask = (MC_BACK | MC_TEXT | MC_SEL_BACK | MC_SEL_TEXT | MC_FRAME);			
	mclr.cBack = RGB_WHITE;
	mclr.cText = RGB_BLACK;
	mclr.cSelBack = MAKE_RGB (146, 109, 255);
	mclr.cSelText = RGB_BLACK;
	mclr.cFrame = RGB_WHITE;

	IMENUCTL_SetColors (pMe->m_pIMenu, &mclr);
		
	// ޴ Ʈ ŸƲ Ѵ
	STREXPAND((byte *)"TAPI/SMS", STRLEN("TAPI/SMS"), szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);
	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  
	
	STREXPAND((byte *)"1. ȭ ɱ", STRLEN("1. ȭ ɱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_MAKEVOICECALL, szBuf, 0);
	
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}
static boolean TapiExample_InitAppData(IApplet* pi)
{
	CTapiExample* pMe = (CTapiExample*)pi;
	int pnAscent = 0;
	int pnDescent = 0;

	pMe->m_pIMenu = NULL;
	pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);
	/*  */
	pMe->status = EXAMPLE_NORMAL;
	return TRUE;
}
static void TapiExample_FreeAppData(IApplet* pi)
{
	CTapiExample* pMe = (CTapiExample*)pi;
	if (pMe->m_pIMenu != NULL)
	{
		// ޴ Ʈ ϴ Լ
		IMENUCTL_Release (pMe->m_pIMenu);
		pMe->m_pIMenu = NULL;
	}
}

static void DisplayOutput(IApplet * pi, int nline, char *pszStr, RGBVAL clrColor)
{
	AEEDeviceInfo di;        // Device Info
	AECHAR szBuf[200] = {0}; // a buffer that supports 200 char string
	AECHAR * psz = NULL;
	int pixelWidth;
	AEEFont font = AEE_FONT_NORMAL;
	int pnFits = 0, dy;
	int totalCh = 0;
	int charHeight = 0;      // Stores the char height in pixels for given font
	int pnAscent = 0;        // Stores the ascent in number of pixels
	int pnDescent = 0;       // Stores the descent in number of pixels
	AEERect rc;

	AEEApplet * pMe = (AEEApplet*)pi;

	if (pMe == NULL)
		return;
	   
	// Get device information
	ISHELL_GetDeviceInfo(pMe->m_pIShell,&di);

	// Get the font metrics info
	charHeight = IDISPLAY_GetFontMetrics (pMe->m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

	// Convert to wide string (unicode)
	// STR_TO_WSTR ((char *)pszStr, szBuf, sizeof (szBuf));
   
	STREXPAND((byte *)pszStr, STRLEN(pszStr), szBuf, sizeof(szBuf));
   
	//STREXPAND("ISHELL ", sizeof("ISHELL "), szBuf, sizeof(szBuf));
	// If nlines is less than zero then print this string starting around the middle of 
	// the screen. Or else multiply nlines by 10 to decide the y coordinate of
	// the start of the string.
	if (nline < 0) {
		dy = di.cyScreen*2/5;
	}
	else{
		dy = nline * charHeight;
	}

	// psz keeps track of the point from which to write from the string buffer
	// in case the string does not fit one line and needs to wrap around in the
	// next line.
	psz = szBuf;
      
	// Need to calculate the lotal string length to decide if any wrapping
	// around is needed.
	totalCh = STRLEN ((char *)pszStr);

	// Keep displaying text string on multiple lines if the string can't be displayed
	// on one single line. Lines are spaced 15 pixels apart.
	while ((totalCh > 0) && (*psz != NULL))
	{ 
		// Get information on how many characters will fit in a line.
		// Give the pointer to the buffer to be displayed, and the number of
		// pixels along the x axis you want to display the string in (max number)
		// pnFits will have the max number of chars that will fit in the maxWidth
		// number of pixels (given string can't fit in one line), or the number of 
		// chars in the string (if it does fit in one line). pnWidth gives the
		// number of pixels that will be used to display pnFits number of chars.
		pixelWidth = IDISPLAY_MeasureTextEx(pMe->m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

		// If pnFits is zero there is something wrong in the input to above function. 
		// Normally this scenario should not occur. But, have the check anyway.
		if (pnFits == 0)
			return;

		SETAEERECT(&rc, 0, dy, di.cxScreen, charHeight);
		IDISPLAY_EraseRect(pMe->m_pIDisplay, &rc);
		IDISPLAY_SetColor(pMe->m_pIDisplay,CLR_USER_TEXT,clrColor);
        IDISPLAY_DrawText(pMe->m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

		psz += pnFits;      // move pointer to the next segment to be displayed
		totalCh -= pnFits;  // reduce the total number of characters to still display
		dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line. 
		IDISPLAY_Update(pMe->m_pIDisplay);
		if (totalCh < pnFits)
			pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
	}
	return;  
}
