/*===========================================================================

FILE: filesample.c
===========================================================================*/


/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"          // Module interface definitions
#include "AEEAppGen.h"          // Applet interface definitions
#include "AEEShell.h"           // Shell interface definitions
#include "AEEFile.h"			// File interface definitions
#include "AEEStdLib.h"
#include "AEEMenu.h"
#include "AEEDisp.h"

#include "filesample.bid"

#define MENU_EVT_FILE		401
#define	MENU_EVT_DIRECTORY	402
#define MENU_EVT_DELFILE	403
#define MENU_EVT_DELDIR		404
#define MENU_EVT_RENAME		405
#define MENU_EVT_FILERW		406

typedef struct _CIFileSample {
   AEEApplet	      a;
   IMenuCtl *  m_pIMenu;
   IFileMgr * m_pIFileMgr;
   AEERect     m_rc;
   int m_cLineNum; // screen line to be written to next
   int m_cMaxLine; // max line number of screen on which output can be written
   int             m_nLineHeight;
   AEEDeviceInfo   m_dInfo;
} CIFileSample;

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static boolean filesample_HandleEvent(IApplet * pi, AEEEvent eCode, 
                                      uint16 wParam, uint32 dwParam);

static void DisplayMenu(CIFileSample *pMe);
static boolean IFileSample_InitAppData(IApplet* pi);
static void IFileSample_FreeAppData(IApplet* pi);
static void DisplayOutput(IApplet * pMe, int nline, char *pszStr, RGBVAL clrColor);

/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDisplay_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDisplay_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   if(ClsId == AEECLSID_FILESAMPLE){
      if(AEEApplet_New(sizeof(CIFileSample), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)filesample_HandleEvent,(PFNFREEAPPDATA)IFileSample_FreeAppData)
         == TRUE)
      {
		 // Add your code here .....
		if (IFileSample_InitAppData((IApplet*)*ppObj) == TRUE)
			return (AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION filesample_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean filesample_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean filesample_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
	CIFileSample * pMe = (CIFileSample *)pi;
	
   switch (eCode) 
	{
      case EVT_APP_START:                        
		    
		    // Add your code here .....
		    DisplayMenu(pMe);
      		return(TRUE);
      case EVT_APP_STOP:

		    // Add your code here .....
         return TRUE;
	  case EVT_KEY:
			if (pMe->m_pIMenu)
			{
				if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) && (wParam == AVK_RIGHT))
				{
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw(pMe->m_pIMenu);
				}
			    return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
			}
			else 
				return FALSE;
	  case EVT_COMMAND:
		  {
			  switch(wParam)
			  {
				  case MENU_EVT_FILE:
					{
						IFile * pIFile;
					  
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						DisplayOutput((IApplet *)pMe, 1, " ", MAKE_RGB(0,0,0));
					  
						if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
						  return TRUE;

						if ((pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "kamja.txt", _OFM_CREATE)) == NULL)
						{
							DisplayOutput((IApplet *)pMe, 4, "  ", MAKE_RGB(0,0xff,0));
						}
						else
						{
							DisplayOutput((IApplet *)pMe, 4, "kamja.txt   ", MAKE_RGB(0,0xff,0));
							IFILE_Release(pIFile);
						}
						IFILEMGR_Release(pMe->m_pIFileMgr);
						pMe->m_pIFileMgr = NULL;
						return TRUE;
					}
				  
				  case MENU_EVT_DIRECTORY:
					{
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
					    IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
					    IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
					    DisplayOutput((IApplet *)pMe, 1, " ", MAKE_RGB(0,0,0));
					    
						if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
							return TRUE;
						if (IFILEMGR_MkDir(pMe->m_pIFileMgr, "kamja") != SUCCESS)
							DisplayOutput((IApplet *)pMe, 4, "丮  ", MAKE_RGB(0,0xff,0));
						else
							DisplayOutput((IApplet *)pMe, 4, "丮  ", MAKE_RGB(0,0xff,0));
						IFILEMGR_Release(pMe->m_pIFileMgr);
						pMe->m_pIFileMgr = NULL;
						return TRUE;
					}
				  case MENU_EVT_DELFILE:
					{
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
					    IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
					    IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
					    DisplayOutput((IApplet *)pMe, 1, "kamja.txt  ", MAKE_RGB(0,0,0));
						if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
							return TRUE;
						if(IFILEMGR_Remove(pMe->m_pIFileMgr, "kamja.txt") != SUCCESS)
							DisplayOutput((IApplet *)pMe, 4, "ش   ʾƿ", MAKE_RGB(0,0xff,0));
						else
							DisplayOutput((IApplet *)pMe, 4, "kamja.txt  ߽ϴ.", MAKE_RGB(0,0xff,0));
						IFILEMGR_Release(pMe->m_pIFileMgr);
						pMe->m_pIFileMgr = NULL;
						return TRUE;
					}
				  case MENU_EVT_DELDIR:
					{
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
					    IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
					    IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
					    DisplayOutput((IApplet *)pMe, 1, "kamja  ", MAKE_RGB(0,0,0));
						if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
							return TRUE;
						if (IFILEMGR_RmDir(pMe->m_pIFileMgr, "kamja") != SUCCESS)
							DisplayOutput((IApplet *)pMe, 4, "ش   ʾƿ", MAKE_RGB(0,0xff,0));
						else
							DisplayOutput((IApplet *)pMe, 4, "kamja  ߽ϴ.", MAKE_RGB(0,0xff,0));
						IFILEMGR_Release(pMe->m_pIFileMgr);
						pMe->m_pIFileMgr = NULL;
						return TRUE;		    
					}
				  case MENU_EVT_RENAME:
					{					
						// IFILEMGR_EnumNext Լ о   Ѵ.
						FileInfo fi;
						char szBuf[100];
						// ȭ  ޴ Ʈ ȰȭŲ.
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						// IFileMgr ̽ ü Ѵ.						
						if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
						{
							// ü    ȭ鿡  ޽ Ѵ.
							DisplayOutput((IApplet *)pMe, 1, "IFileMgr ̽  ", MAKE_RGB(0, 0, 0xff));
							return TRUE;
						}
						//  IFileMgr ̽ ̿ؼ ϸ ϴ IFILEMGR_Rename Լ ȣ// Ѵ.
						if (IFILEMGR_Rename(pMe->m_pIFileMgr, "kamja.txt", "kamjanew.txt") != SUCCESS)
						{
							// Լ ȣ    ޽ Ѵ.
							DisplayOutput((IApplet *)pMe, 1, "kamja.txt  .(IFILEMGR_Rename)", MAKE_RGB(0, 0, 0xff));
							//  IFileMgr ̽ Ѵ
							IFILEMGR_Release(pMe->m_pIFileMgr);
							pMe->m_pIFileMgr = NULL;
							return TRUE;
						}
						// IFILEMGR_EnumInit Լ ̿ؼ ""(Ʈ) 丮 Ʒ ִ   о//  غ Ѵ. 
						if(IFILEMGR_EnumInit(pMe->m_pIFileMgr, "", FALSE) != SUCCESS)
						{
							// Լ ȣ    ޽ Ѵ.
							DisplayOutput((IApplet *)pMe, 1, "IFILEMGR_EnumInit function fail", MAKE_RGB(0, 0, 0xff));
							IFILEMGR_Release(pMe->m_pIFileMgr);
							pMe->m_pIFileMgr = NULL;
							return TRUE;
						}
						// ""(Ʈ) 丮 ִ   ϳ  оδ.
						while (IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi))
						{
							//  츮 ã ִ 'kamjanew.txt'  ִٸ 'kamjanew.txt'    // ȭ鿡 Ѵ.
							if(!STRCMP("kamjanew.txt", fi.szName))
							{
								DisplayOutput((IApplet *)pMe, 1, "kamjanew.txt  ãҽϴ", MAKE_RGB(0,0,0));
								SPRINTF(szBuf, " ̸ : %s", fi.szName);
								DisplayOutput((IApplet *)pMe, 4 , szBuf, MAKE_RGB(0, 0xff, 0));
								SPRINTF(szBuf, " ũ : %d", fi.dwSize);
								DisplayOutput((IApplet *)pMe, 6, szBuf, MAKE_RGB(0, 0xff, 0));
								SPRINTF(szBuf, "  ð : %d", fi.dwCreationDate);
								DisplayOutput((IApplet *)pMe, 8 , szBuf, MAKE_RGB(0, 0xff, 0));
								switch(fi.attrib)
								{
								case _FA_NORMAL:
									DisplayOutput((IApplet *)pMe, 9 , " Ӽ : _FA_NORMAL", MAKE_RGB(0, 0xff, 0));
									break;
								case _FA_HIDDEN:
									DisplayOutput((IApplet *)pMe, 9 , " Ӽ : _FA_HIDDEN", MAKE_RGB(0, 0xff, 0));
									break;
								case _FA_DIR:
									DisplayOutput((IApplet *)pMe, 9 , " Ӽ : _FA_DIR", MAKE_RGB(0, 0xff, 0));
									break;
								case _FA_READONLY:
									DisplayOutput((IApplet *)pMe, 9 , " Ӽ : _FA_READONLY", MAKE_RGB(0, 0xff, 0));
									break;
								case _FA_SYSTEM:
									DisplayOutput((IApplet *)pMe, 9 , " Ӽ : _FA_SYSTEM", MAKE_RGB(0, 0xff, 0));
									break;
								default:
									break;
								}
							}
						}	
					IFILEMGR_Release(pMe->m_pIFileMgr);
					pMe->m_pIFileMgr = NULL;
					return TRUE;
					}
				case MENU_EVT_FILERW:
				{
					IFile * pIFile;
					FileInfo pi;
					char pszStr[]  = "   ִ";
					char pszStrw[] = " ";
					char szBuffer[50] = {0};
					// ȭ Ѵ
					IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
					IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
					IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
												
					DisplayOutput((IApplet *)pMe, 1 , " а ", MAKE_RGB(0, 0, 0xff));
					// IFileMgr ̽ ü Ѵ.
					if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR,(void **)&pMe->m_pIFileMgr))
						return TRUE;
					// sample.txt  ִ  ˻Ѵ. 
					//  ϸ IFILEMGR_Remove Լ ̿ؼ Ѵ.		
					if(IFILEMGR_GetInfo(pMe->m_pIFileMgr, "sample.txt", &pi) == SUCCESS)
					{
						IFILEMGR_Remove(pMe->m_pIFileMgr, "sample.txt");
					}
								
					//  Ѵ.
					if ((pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, "sample.txt", _OFM_CREATE)) == NULL)
					{
						IFILEMGR_Release(pMe->m_pIFileMgr);
						pMe->m_pIFileMgr = NULL;
						return TRUE;
					}
					// Ͽ  .
					IFILE_Write(pIFile, pszStr, sizeof(pszStr));
					IFILE_Write(pIFile, pszStrw, sizeof(pszStrw));
					// Ͽ  о   ͸   Ű ´
					if (IFILE_Seek(pIFile, _SEEK_START, 0) == SUCCESS)
					{
						// Ͽ  ŭ  о´.
						if (IFILE_Read(pIFile, szBuffer, sizeof(pszStr)) == sizeof(pszStr))
							DisplayOutput((IApplet *)pMe, 4 , szBuffer, MAKE_RGB(0, 0, 0xff));
						if (IFILE_Read(pIFile, szBuffer, sizeof(pszStrw)) == sizeof(pszStrw))
							DisplayOutput((IApplet *)pMe, 6 , szBuffer, MAKE_RGB(0, 0, 0xff));
					}
					//  ݴ´.
					IFILE_Release(pIFile);
					// IFileMgr ̽ ü Ѵ.
					IFILEMGR_Release(pMe->m_pIFileMgr); 
					pMe->m_pIFileMgr = NULL;
					return TRUE;
				 }

				  default:
					  return TRUE;
			  }		
		  }
	  default:
         break;
   }
   return FALSE;
}


static void DisplayMenu(CIFileSample *pMe)
{
	AEERect rc;
	AECHAR  szBuf[50];
	char    szStr[50];
	AEEMenuColors mclr;
	int n ;
	
	if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu) != SUCCESS)
		return;
	
	// Set display rectangle
	SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

	// õ ޴  Ѵ. 
	mclr.wMask = (MC_BACK | MC_TEXT | MC_SEL_BACK | MC_SEL_TEXT | MC_FRAME);			
	mclr.cBack = RGB_WHITE;
	mclr.cText = RGB_BLACK;
	mclr.cSelBack = MAKE_RGB (146, 109, 255);
	mclr.cSelText = RGB_BLACK;
	mclr.cFrame = RGB_WHITE;

	IMENUCTL_SetColors (pMe->m_pIMenu, &mclr);
		
	// ޴ Ʈ ŸƲ Ѵ
	STREXPAND((byte *)"File, FileMgr ", STRLEN("File, FileMgr "), szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);
	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  
	
	STREXPAND((byte *)"1.  ϱ", STRLEN("1.  ϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_FILE, szBuf, 0);
	
	STREXPAND((byte *)"2. 丮 ϱ", STRLEN("2. 丮 ϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_DIRECTORY, szBuf, 0);

	STREXPAND((byte *)"3. kamja.txt  ", STRLEN("3. kamja.txt  "), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_DELFILE, szBuf, 0);
	
	STREXPAND((byte *)"4. kamja  ", STRLEN("4. kamja  "), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_DELDIR, szBuf, 0);
	
	STREXPAND((byte *)"5.  ̸ ", STRLEN("5.  ̸ "), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_RENAME, szBuf, 0);
	
	STREXPAND((byte *)"6.  а ", STRLEN("6.  а "), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_FILERW, szBuf, 0);

	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

static boolean IFileSample_InitAppData(IApplet* pi)
{
	CIFileSample* pMe = (CIFileSample*)pi;
	int pnAscent = 0;
	int pnDescent = 0;
	// Initialize the MenuCtl pointer to NULL
	pMe->m_pIMenu = NULL;
	pMe->m_pIFileMgr = NULL;
	pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);
	return TRUE;

}
static void IFileSample_FreeAppData(IApplet* pi)
{
	CIFileSample* pMe = (CIFileSample*)pi;
	// ޴ Ʈ   ʾҴٸ Ѵ.
	if (pMe->m_pIMenu != NULL)
	{
		// ޴ Ʈ ϴ Լ
		IMENUCTL_Release (pMe->m_pIMenu);
		pMe->m_pIMenu = NULL;
	}
	if(pMe->m_pIFileMgr != NULL)
    {
	  IFILEMGR_Release(pMe->m_pIFileMgr);
      pMe->m_pIFileMgr = NULL;
    }
}

static void DisplayOutput(IApplet * pi, int nline, char *pszStr, RGBVAL clrColor)
{
   AEEDeviceInfo di;        // Device Info
   AECHAR szBuf[200] = {0}; // a buffer that supports 200 char string
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
	AEERect rc;

   AEEApplet * pMe = (AEEApplet*)pi;

   if (pMe == NULL)
      return;
	   
   // Get device information
   ISHELL_GetDeviceInfo(pMe->m_pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pMe->m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   // STR_TO_WSTR ((char *)pszStr, szBuf, sizeof (szBuf));
   
   STREXPAND((byte *)pszStr, STRLEN(pszStr), szBuf, sizeof(szBuf));
   
   //STREXPAND("ISHELL ", sizeof("ISHELL "), szBuf, sizeof(szBuf));
   // If nlines is less than zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
         return;

		SETAEERECT(&rc, 0, dy, di.cxScreen, charHeight);
		IDISPLAY_EraseRect(pMe->m_pIDisplay, &rc);
		IDISPLAY_SetColor(pMe->m_pIDisplay,CLR_USER_TEXT,clrColor);
        IDISPLAY_DrawText(pMe->m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line. 
      IDISPLAY_Update(pMe->m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }
   return;   
}
