/*===========================================================================

FILE: idisplaysample.c
===========================================================================*/


/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"          // Module interface definitions
#include "AEEAppGen.h"          // Applet interface definitions
#include "AEEShell.h"           // Shell interface definitions
#include "AEEStdLib.h"
#include "AEEMenu.h"
#include "AEEDisp.h"

#include "idisplaysample.bid"
#include "idisplaysample_res.h"

#define MENU_EVT_DISPLAY		401
#define MENU_EVT_BITBLT			402
#define MENU_EVT_FRAMEBUTTON	403
#define MENU_EVT_DRAWRECT		404
#define MENU_EVT_DRAWFRAME		405
#define MENU_EVT_DRAWLINE		406

#define  APP_RES_FILE      "idisplaysample.bar"
typedef struct _CIDisplaySample
{
	AEEApplet	a;        // Mandatory first AEEApplet data member
	IMenuCtl *  m_pIMenu;

	// Device parameters
	int             m_nLineHeight;
	int				m_nButton;
	int				m_nDrawRect;
	int				m_nFrame;
	int				m_nDrawLine;
	AEEDeviceInfo   m_dInfo;

} CIDisplaySample;

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static boolean idisplaysample_HandleEvent(IApplet * pi, AEEEvent eCode, 
                                      uint16 wParam, uint32 dwParam);
static void DisplayOutput(IApplet * pMe, int nline, char *pszStr, RGBVAL clrColor);
static void DisplayMenu(CIDisplaySample *pMe);
static boolean IDisplaySample_InitAppData(IApplet* pi);
static void IDisplaySample_FreeAppData(IApplet* pi);
static void DisplayFrameButton(CIDisplaySample *pMe, boolean buttonType);
/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDisplay_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDisplay_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
	*ppObj = NULL;
		
   if(ClsId == AEECLSID_IDISPLAYSAMPLE){
      if(AEEApplet_New(sizeof(CIDisplaySample), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)idisplaysample_HandleEvent,(PFNFREEAPPDATA)IDisplaySample_FreeAppData)
         == TRUE)
      {
		 // Add your code here .....
		 if (IDisplaySample_InitAppData((IApplet*)*ppObj) == TRUE)
			return (AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION idisplaysample_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean idisplaysample_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean idisplaysample_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
	
	CIDisplaySample * pMe = (CIDisplaySample *)pi;
	
	switch (eCode) 
	{
		case EVT_APP_START:                        
		    DisplayMenu(pMe);
      		return(TRUE);
		case EVT_APP_STOP:
			
		    return TRUE;
		case EVT_KEY:
			if (pMe->m_pIMenu)
			{
				if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) && (wParam == AVK_RIGHT))
				{
					pMe->m_nButton = 0;
					pMe->m_nFrame = 0;
					pMe->m_nDrawRect = 0;
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw(pMe->m_pIMenu);
				}
			    return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
			}
			else 
				return FALSE;
		case EVT_KEY_PRESS:
			{
				if((pMe->m_nButton == 1) && (wParam == AVK_SELECT))
				{
					DisplayFrameButton(pMe, FALSE);
				}
				if((pMe->m_nDrawRect >= 2) && (wParam == AVK_SELECT))
				{
					AEERect rct = {0};
					AEEFrameType fType;
					RGBVAL clrFrame, clrFill;
					uint32 dwFlags;

					switch(pMe->m_nDrawRect)
					{
					case 2:
					{
						SETAEERECT (&rct, 5, pMe->m_nLineHeight*3 + 4, pMe->m_dInfo.cxScreen/2, pMe->m_nLineHeight); 
						clrFrame = MAKE_RGB (0, 0, 0);             // Black color
						clrFill =  MAKE_RGB (0, 0xff, 0) ;		// White color
						// Draw only the outline of the rectangle (rectangle frame).
						dwFlags = IDF_RECT_INVERT;
						pMe->m_nDrawRect += 1;
						IDISPLAY_DrawRect(pMe->a.m_pIDisplay, &rct, clrFrame, clrFill, dwFlags);
						IDISPLAY_Update(pMe->a.m_pIDisplay);
						break;
					}
					case 3:
					{
						SETAEERECT (&rct, 5, pMe->m_nLineHeight*4 + 6, pMe->m_dInfo.cxScreen/2, pMe->m_nLineHeight);
						clrFrame = MAKE_RGB (0, 0xff, 0);         // Green color
						clrFill =  MAKE_RGB (0xff, 0, 0);         // Red color
						dwFlags = IDF_RECT_FILL;
						pMe->m_nDrawRect += 1;
						IDISPLAY_DrawRect(pMe->a.m_pIDisplay, &rct, clrFrame, clrFill, dwFlags);
						IDISPLAY_Update(pMe->a.m_pIDisplay);
						break;
					}
					case 4:
					{
						SETAEERECT (&rct, 5, pMe->m_nLineHeight*5 + 8, pMe->m_dInfo.cxScreen/2, pMe->m_nLineHeight);
						clrFrame = MAKE_RGB (0, 0, 0xff);   // Blue color
						clrFill =  MAKE_RGB (0, 0xff, 0);   // Green color
						dwFlags = IDF_RECT_FRAME | IDF_RECT_FILL;
						pMe->m_nDrawRect += 1;
						IDISPLAY_DrawRect(pMe->a.m_pIDisplay, &rct, clrFrame, clrFill, dwFlags);
						IDISPLAY_Update(pMe->a.m_pIDisplay);
						break;
					}
					case 5:
					{
						SETAEERECT (&rct, 5, pMe->m_nLineHeight*6 + 10, pMe->m_dInfo.cxScreen/2, pMe->m_nLineHeight);
						clrFrame = MAKE_RGB (0, 0, 0xff);   // Blue color
						clrFill =  MAKE_RGB (0, 0xff, 0);   // Green color
						dwFlags = IDF_RECT_FRAME;
						pMe->m_nDrawRect += 1;
						IDISPLAY_DrawRect(pMe->a.m_pIDisplay, &rct, clrFrame, clrFill, dwFlags);
						IDISPLAY_Update(pMe->a.m_pIDisplay);
						break;
					}
					default:
						break;
					}
					
				}
				if((pMe->m_nFrame >= 1) && (wParam == AVK_SELECT))
				{
					AEERect rct;
					AEEFrameType fType;
					RGBVAL clrFill;
					if(pMe->m_nFrame >=6)
					{
						SETAEERECT (&rct, 5, pMe->m_nLineHeight*(2*(pMe->m_nFrame-5)+1), pMe->m_dInfo.cxScreen/2, pMe->m_nLineHeight);
						clrFill = MAKE_RGB (0, 0xff, 0);
					}
					else
					{
						SETAEERECT (&rct, 5, pMe->m_nLineHeight*(2*pMe->m_nFrame+1), pMe->m_dInfo.cxScreen/2, pMe->m_nLineHeight);
						clrFill = MAKE_RGB (0, 0, 0xff);
					}
					switch(pMe->m_nFrame)
					{
					case 1:
					case 6:
						if(pMe->m_nFrame == 1)
						{
							DisplayOutput((IApplet *)pMe, 2*pMe->m_nFrame, "AEE_FT_3D_EMPTY", MAKE_RGB(0,0,0));
							fType = AEE_FT_3D_EMPTY;
						}
						else
						{
							DisplayOutput((IApplet *)pMe, (pMe->m_nFrame-5)*2, "AEE..BOTTOM_SEL", MAKE_RGB(0,0,0));
							fType = AEE_FT_TAB_BOTTOM_SEL;
						}
						IDISPLAY_DrawFrame (pMe->a.m_pIDisplay, &rct, fType, clrFill);
						IDISPLAY_Update(pMe->a.m_pIDisplay);
						pMe->m_nFrame += 1;
						break;
					case 2:
					case 7:
						if(pMe->m_nFrame == 2)
						{
							DisplayOutput((IApplet *)pMe, pMe->m_nFrame*2, "AEE_FT_RAISED", MAKE_RGB(0,0,0));
							fType = AEE_FT_RAISED;
						}
						else
						{
							DisplayOutput((IApplet *)pMe, (pMe->m_nFrame-5)*2, "AEE..TAB_BOTTOM", MAKE_RGB(0,0,0));
							fType = AEE_FT_TAB_BOTTOM;
						}
						IDISPLAY_DrawFrame (pMe->a.m_pIDisplay, &rct, fType, clrFill);
						IDISPLAY_Update(pMe->a.m_pIDisplay);
						pMe->m_nFrame += 1;
						break;
					case 3:
					case 8:
						if(pMe->m_nFrame == 3)
						{
							DisplayOutput((IApplet *)pMe, pMe->m_nFrame*2, "AEE_FT_LOWERED", MAKE_RGB(0,0,0));
							fType = AEE_FT_LOWERED;
						}
						else
						{
							DisplayOutput((IApplet *)pMe, (pMe->m_nFrame-5)*2, "AEE_FT_TAB_TOP", MAKE_RGB(0,0,0));
							fType = AEE_FT_TAB_TOP;
						}
						IDISPLAY_DrawFrame (pMe->a.m_pIDisplay, &rct, fType, clrFill);
						IDISPLAY_Update(pMe->a.m_pIDisplay);
						pMe->m_nFrame += 1;
						break;
					case 4:
					case 9:
						if(pMe->m_nFrame == 4)
						{
							DisplayOutput((IApplet *)pMe, pMe->m_nFrame*2, "AEE_FT_BOX", MAKE_RGB(0,0,0));
							fType = AEE_FT_BOX;
						}
						else
						{
							DisplayOutput((IApplet *)pMe, (pMe->m_nFrame-5)*2, "AEE_FT_TAB_TOP_SEL", MAKE_RGB(0,0,0));
							fType = AEE_FT_TAB_TOP_SEL;
							IDISPLAY_EraseRgn (pMe->a.m_pIDisplay, 0, pMe->m_nLineHeight*10,
                               pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);
						}
						IDISPLAY_DrawFrame (pMe->a.m_pIDisplay, &rct, fType, clrFill);
						IDISPLAY_Update(pMe->a.m_pIDisplay);
						pMe->m_nFrame += 1;
						break;
					case 5:
						DisplayOutput((IApplet *)pMe, pMe->m_nFrame*2, "AEE_FT_INDENT", MAKE_RGB(0,0,0));
						fType = AEE_FT_INDENT;
						IDISPLAY_DrawFrame (pMe->a.m_pIDisplay, &rct, fType, clrFill);
						IDISPLAY_Update(pMe->a.m_pIDisplay);
						pMe->m_nFrame += 1;
						break;
					default:
						break;
					}
				}
				if((pMe->m_nDrawLine >= 1) && (wParam == AVK_SELECT))
				{
					int16 startX = 10;
					int16 startY = pMe->m_dInfo.cyScreen/4;
					int16 len = pMe->m_dInfo.cxScreen/2;
					switch(pMe->m_nDrawLine)
					{
					case 1:
						IDISPLAY_DrawHLine(pMe->a.m_pIDisplay, startX, startY, len);
						break;
					case 2:
						IDISPLAY_DrawVLine(pMe->a.m_pIDisplay, startX, startY, len);
						break;
					case 3:
						IDISPLAY_DrawHLine(pMe->a.m_pIDisplay, startX, startY+len, len);
						break;
					case 4:
						IDISPLAY_DrawVLine(pMe->a.m_pIDisplay, startX + len, startY, len);
						break;
					default:
						break;
					}
					pMe->m_nDrawLine += 1;
					IDISPLAY_Update(pMe->a.m_pIDisplay);
					//DisplayFrameButton(pMe, FALSE);
				}
				return TRUE;
			}
		case EVT_KEY_RELEASE:
			{
				if((pMe->m_nButton == 1) && (wParam == AVK_SELECT))
				{
					DisplayFrameButton(pMe, TRUE);
				}
				
			    return TRUE;

			}
		case EVT_COMMAND:
			{
				switch(wParam)
				{
				case MENU_EVT_DISPLAY:
					{
					IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
					IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
					IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
					DisplayOutput((IApplet *)pMe, 1, "ù° ϱ", MAKE_RGB(0xff,0,0));
					DisplayOutput((IApplet *)pMe, 2, "ι° ϱ", MAKE_RGB(0,0xff,0));
					DisplayOutput((IApplet *)pMe, 3, "° ϱ", MAKE_RGB(0,0,0xff));
					DisplayOutput((IApplet *)pMe, 4, "׹° ϱ", MAKE_RGB(0,0,0));
					return TRUE;
					}
				case MENU_EVT_BITBLT:
				{
					AEEBmp pbmSource;
					int xDest = 0;
					int yDest = 10;
					int cxDest = pMe->m_dInfo.cxScreen;
					int cyDest = pMe->m_dInfo.cyScreen;
					int xSrc = 0;
					int ySrc = 0;
					AEERasterOp dwRopCode = AEE_RO_MASK;
					AEEImageInfo imageInfo;
					void *pBmp = NULL, *pDataBytes = NULL;
					boolean bVal = TRUE;

					IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
					IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
					IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
					// Load bitmap resource from the resource file.
					pbmSource = ISHELL_LoadResData (pMe->a.m_pIShell, APP_RES_FILE, 
                                     IDB_BITMAP, RESTYPE_IMAGE); // please do not use RESTYPE_BINARY

					//There is a header that needs to be removed before passing the data to convertbmp
					//the first byte specifies the size of the header and hence offset  we need to skip.
					pDataBytes = (byte *)pbmSource + *((byte *)pbmSource);


					pBmp = CONVERTBMP (pDataBytes, &imageInfo, &bVal);

					// IDISPLAY_BitBlt: performs a bit-block transfer of the data 
					// corresponding to a rectangle of pixels from the specified source 
					// bitmap into the given Display area. After calling this function, 
					// IDISPLAY_Update() must be called to update the screen. If there is 
					// a sequence of drawing operations being performed, it is sufficient 
					// to IDISPLAY_Update() once after all the drawing is done.
					IDISPLAY_BitBlt (pMe->a.m_pIDisplay, xDest, yDest, cxDest, cyDest, pBmp,
							xSrc, ySrc, dwRopCode);

					IDISPLAY_Update (pMe->a.m_pIDisplay);
					ISHELL_FreeResData (pMe->a.m_pIShell, pbmSource);
					if(bVal)//free only if realloc was done
						SYSFREE ( pBmp);
					return TRUE;
				}
				case MENU_EVT_FRAMEBUTTON:
					{
					
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						
						DisplayOutput((IApplet *)pMe, 1, " ư  ^^", MAKE_RGB(0,0,0));
						DisplayOutput((IApplet *)pMe, 8, " ޴  Ű ϼ.", MAKE_RGB(0,0,0));
						DisplayFrameButton(pMe, TRUE);
						pMe->m_nButton = 1;
						return TRUE;
					}
				case MENU_EVT_DRAWRECT:
					{
												
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						DisplayOutput((IApplet *)pMe, 1, "DrawRect Լ", MAKE_RGB(0,0,0));
						// 簢 ׸ 
						pMe->m_nDrawRect = 2;
						return TRUE;
					}
				case MENU_EVT_DRAWFRAME:
					{
						/*AEERect rct;
						AEEFrameType fType;
						RGBVAL clrFill;
						SETAEERECT (&rct, 5, pMe->m_nLineHeight*2 + 2, pMe->m_dInfo.cxScreen/2, pMe->m_nLineHeight);
						fType = AEE_FT_EMPTY;
						clrFill = MAKE_RGB (0, 0, 0xff);
						
						
						IDISPLAY_DrawFrame (pMe->a.m_pIDisplay, &rct, fType, clrFill);
						IDISPLAY_Update(pMe->a.m_pIDisplay);
						*/
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						DisplayOutput((IApplet *)pMe, 1, "DrawFrame Լ", MAKE_RGB(0,0,0));
						IDISPLAY_Update(pMe->a.m_pIDisplay);
						pMe->m_nFrame = 1;
						return TRUE;
					}
				case MENU_EVT_DRAWLINE:
					{
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						DisplayOutput((IApplet *)pMe, 1, " ׸ Լ", MAKE_RGB(0,0,0));
						IDISPLAY_Update(pMe->a.m_pIDisplay);
						pMe->m_nDrawLine = 1;
						return TRUE;
					}
				default:
					return FALSE;
				}
			}
      default:
         break;
   }
   return FALSE;
}

static boolean IDisplaySample_InitAppData(IApplet* pi)
{
	CIDisplaySample* pMe = (CIDisplaySample*)pi;
	int pnAscent = 0;
	int pnDescent = 0;
	// Initialize the MenuCtl pointer to NULL
	pMe->m_pIMenu = NULL;
	pMe->m_nButton = 0;
	pMe->m_nFrame = 0;
	pMe->m_nDrawLine = 0;
	pMe->m_nDrawRect = 0;

	pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);
	return TRUE;
}

static void IDisplaySample_FreeAppData(IApplet* pi)
{   
	CIDisplaySample* pMe = (CIDisplaySample*)pi;
	// ޴ Ʈ   ʾҴٸ Ѵ.
	if (pMe->m_pIMenu != NULL)
	{
		// ޴ Ʈ ϴ Լ
		IMENUCTL_Release (pMe->m_pIMenu);
		pMe->m_pIMenu = NULL;
	}
}

static void DisplayMenu(CIDisplaySample *pMe)
{
	AEERect rc;
	AECHAR  szBuf[50];
	char    szStr[50];
	AEEMenuColors mclr;
	int n ;
	
	if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu) != SUCCESS)
		return;
	
	// Set display rectangle
	SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

	// õ ޴  Ѵ. 
	mclr.wMask = (MC_BACK | MC_TEXT | MC_SEL_BACK | MC_SEL_TEXT | MC_FRAME);			
	mclr.cBack = RGB_WHITE;
	mclr.cText = RGB_BLACK;
	mclr.cSelBack = MAKE_RGB (146, 109, 255);
	mclr.cSelText = RGB_BLACK;
	mclr.cFrame = RGB_WHITE;

	IMENUCTL_SetColors (pMe->m_pIMenu, &mclr);
		
	// ޴ Ʈ ŸƲ Ѵ
	//_STR_TO_WSTR("ISHELL ", szBuf, sizeof(szBuf));
	//STREXPAND(const byte * pSrc, int nCount, AECHAR * pDest, int nSize)
		STREXPAND((char *)"IDisplay ", STRLEN("IDisplay "), szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);
	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  
	// ޴ MENU_EVENT_ALARM ̸  ߰Ѵ. 
	// ߰Ǵ  Ŭϸ MENU_EVENT_ALARM   ̺Ʈ 
	// ó  ִ. 
	
	STREXPAND((char *)"1. ؽƮ óϱ", STRLEN("1. ؽƮ óϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_DISPLAY, szBuf, 0);
	
	STREXPAND((char *)"2. Ʈ ̹ óϱ", STRLEN("2. Ʈ ̹ óϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_BITBLT, szBuf, 0);

	STREXPAND((char *)"3. 簢 ׸", STRLEN("4. 簢 ׸"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_DRAWRECT, szBuf, 0);
	
	STREXPAND((char *)"4. ư ϱ", STRLEN("4. ư ϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_FRAMEBUTTON, szBuf, 0);
	
	STREXPAND((char *)"5.  ׸", STRLEN("5.  ׸"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_DRAWFRAME, szBuf, 0);
	
	STREXPAND((char *)"6.  ׸", STRLEN("6.  ׸"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_DRAWLINE, szBuf, 0);
	
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

static void DisplayFrameButton(CIDisplaySample *pMe, boolean buttonType)
{
	AEERect rct;	
	AEEFrameType fType;
	RGBVAL clrFill;
	SETAEERECT (&rct, pMe->m_dInfo.cxScreen/4, pMe->m_nLineHeight*3, pMe->m_dInfo.cxScreen/3, pMe->m_nLineHeight*3/2);
	fType = AEE_FT_EMPTY;
	clrFill = MAKE_RGB (0, 0, 0xff);
	IDISPLAY_FrameButton (pMe->a.m_pIDisplay, &rct, buttonType, clrFill);
	IDISPLAY_Update (pMe->a.m_pIDisplay);
}
static void DisplayOutput(IApplet * pi, int nline, char *pszStr, RGBVAL clrColor)
{
   AEEDeviceInfo di;        // Device Info
   AECHAR szBuf[200] = {0}; // a buffer that supports 200 char string
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
	AEERect rc;

   AEEApplet * pMe = (AEEApplet*)pi;

   if (pMe == NULL)
      return;
	   
   // Get device information
   ISHELL_GetDeviceInfo(pMe->m_pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pMe->m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   // STR_TO_WSTR ((char *)pszStr, szBuf, sizeof (szBuf));
   
   STREXPAND((char *)pszStr, STRLEN(pszStr), szBuf, sizeof(szBuf));
   
   //STREXPAND("ISHELL ", sizeof("ISHELL "), szBuf, sizeof(szBuf));
   // If nlines is less than zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
         return;

		SETAEERECT(&rc, 0, dy, di.cxScreen, charHeight);
		IDISPLAY_EraseRect(pMe->m_pIDisplay, &rc);
		IDISPLAY_SetColor(pMe->m_pIDisplay,CLR_USER_TEXT,clrColor);
        IDISPLAY_DrawText(pMe->m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line. 
      IDISPLAY_Update(pMe->m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }
   return;   
}
