#include "inotiexam.h"

#define NOTIFY_MASK_EVENT          0x00000001    // Dummy event mask for 


uint32  NotifierExam_AddRef(INotifier* po);
uint32  NotifierExam_Release(INotifier* po);
void    NotifierExam_SetMask(INotifier* po, const uint32 * dwMask);
void    NotifierExam_TimerCB (void *pi);

uint32  NotifierExam_AddRef(INotifier* po)
{
   return ++(((NotifierExam *)po)->m_nRefs);
}

uint32  NotifierExam_Release(INotifier* po)
{
	NotifierExam * pMe = (NotifierExam *)po;

	if (--pMe->m_nRefs != 0)
		return pMe->m_nRefs;

	//Release the interfaces created in NotifierExam_New()
   if (pMe->m_pIDisplay)
      IDISPLAY_Release(pMe->m_pIDisplay);   

   // Remove it from the list...
   AEEMod_ListRemove(pMe->m_pIModule, (AEEModObj *)pMe);

   IMODULE_Release(pMe->m_pIModule);

	// delete this NotifierExam object allocated in NotifierExam_New
   FREEOBJ(pMe);   
   
   ISHELL_Release(pMe->m_pIShell);   // Release the Shell

	return 0;
}


void NotifierExam_TimerCB (void *pi)
{
   AEERect rect;       // Set up the rect coords for erasing/displaying
   NotifierExam * pMe = (NotifierExam*)pi;
   AEEDeviceInfo di;  // Device Info

   // Get device infor so that we can erase a section of the screen,
   ISHELL_GetDeviceInfo(pMe->m_pIShell,&di); 
   rect.x = 0;
   rect.y = di.cyScreen/8;
   rect.dx = di.cxScreen;
   rect.dy = di.cyScreen - (di.cyScreen/3);
   
   // First erase the screen where the text is to be printed
   IDISPLAY_EraseRect(pMe->m_pIDisplay,&rect);  //Clear the display

   // "Notifier" signifies the class that provides the event notification
   // services (i.e. TAPI)
   DisplayOutput (pMe->m_pIShell, pMe->m_pIDisplay, 2, "Notifier:Event notified to Shell...");

   // Notify the shell that the requested event has occured on the 
   // Notifier-object.
   ISHELL_Notify(pMe->m_pIShell, AEECLSID_NOTIFIER_APP, 
      pMe->m_dwNotifyMask, (void *) pMe);
}

void NotifierExam_SetMask(INotifier* po, const uint32 * dwMask)
{
   NotifierExam * pMe = (NotifierExam *)po;
   int timerPeriod = 3000;             // Timer period in MS


   // just check if the notifier is the value sent by the user.
   if (*dwMask != NOTIFY_MASK_EVENT)
   {
      DisplayOutput (pMe->m_pIShell, pMe->m_pIDisplay, -1, "Notifier: Mask mismatch");
      return;
   }
   // Store the event that was requested for later use in the callback function.
   pMe->m_dwNotifyMask = *dwMask;

   // Start timer .
   ISHELL_SetTimer (pMe->m_pIShell, timerPeriod, NotifierExam_TimerCB, (void *)pMe);

   // "Notifier" siginifies the applet that provides the event notification
   // services (i.e. TAPI)
   DisplayOutput (pMe->m_pIShell, pMe->m_pIDisplay, 5, "Notifier: Waiting for event to occur (in 3 seconds)...");
}


boolean NotifierExam_New(int16 nSize,
                     AEECLSID clsID,
							IShell * pIShell, 
							IModule * pIModule, 
							NotifierExam **ppobj)
{
   NotifierExam * pMe = NULL;
   VTBL(INotifier) *appFuncs;
		
   *ppobj = NULL;

   //Validate parameters
   if (!pIShell || !pIModule)
      return FALSE;

   if(nSize < sizeof(NotifierExam))
      nSize += sizeof(NotifierExam);

   // Allocate memory for the notifier object and the INotifierVtbl
   if (NULL == (pMe = (NotifierExam*)MALLOC(nSize + sizeof(INotifierVtbl))))
      return FALSE;

   appFuncs = (INotifierVtbl *)((byte *)pMe + nSize);

	//Initialize the individual entries in the VTBL
	appFuncs->AddRef      = NotifierExam_AddRef; 
   appFuncs->Release     = NotifierExam_Release;
   appFuncs->SetMask     = NotifierExam_SetMask;

   INIT_VTBL(pMe, INotifier, *appFuncs);   // Initialize the VTBL

	//Initialize the data members
   pMe->m_nRefs      = 1;
   pMe->m_pIShell    = pIShell;
   pMe->m_pIModule   = pIModule;
   pMe->m_pIDisplay  = NULL;
	pMe->clsID = clsID;

   ISHELL_AddRef(pIShell);
   IMODULE_AddRef(pIModule);

   ISHELL_CreateInstance(pIShell, AEECLSID_DISPLAY, (void **) &pMe->m_pIDisplay);
  	if (!pMe->m_pIDisplay)
	{
		//Cleanup
		FREEOBJ(pMe);
      return FALSE;
	} 
	
   // Add it to the tracking list for the module...
   AEEMod_ListAdd(pIModule, (AEEModObj *)pMe);

   *ppobj = (NotifierExam*)pMe;  

   return TRUE;
}

